
function [Fitness] = FullXOmatlab(PARMS); 
% MODEL NUMBER: 0323
% MODEL NAME: FullXO
% SHORT DESCRIPTION:  Two sequential, first-order enzymatic reactions S <--> P 
% with substrates binding to enzymes, and reversible product formation. Reactions 
% facilitated by a single enzyme, Xanthine Oxidase.


% Data files:
fileHypo = '4.hyp.csv';     % Data rounded to nearest tenth of a second.
fileXan = '4.xan.csv';      % Data rounded to nearest tenth of a second.
fileUric = '4.uric.csv';    % Data rounded to nearest tenth of a second.

% units of time (T): seconds.

% *****************************************************************************
%                        Hx -> Xa --> Ua REACTIONS: 
% *****************************************************************************
 % PARAMETERS
        kfH = PARMS(1); % = 100; % 1/(uM*s);      % Forward reaction binding S to E
        KdH = PARMS(2); %= 2.805; % uM;           % Equil. dissoc. for binding S to E
        kpfH = PARMS(3); %= 1.799; % 1/s;          % Forward reaction for ES to E + P
        KpH = PARMS(4); %= 2000; % uM;           % Equil. dissoc. for binding P to E 
        
        kfX = PARMS(5); %= 100; % 1/(uM*s);      % Forward reaction binding S to E
        KdX = PARMS(6); %= 4.662; % uM;           % Equil. dissoc. for binding S to E
        kpfX = PARMS(7); %= 1.938; % 1/s;          % Forward reaction for ES to E + P
        KpX = PARMS(8); %= 562; % uM;           % Equil. dissoc. for binding P to E 
               
        Etot = PARMS(9);% = 1.0; % uM;            % Total amount of enzyme = 0.27 mg/ml
                                        % MW = 270,000 Daltons 
 % DERIVED PARAMETERS
        kbH  = KdH * kfH;             % 1/s, Backwards reaction ES to S + E
        kpbH = kpfH/KpH;              % 1/(uM*s), Backwards reaction P + E to ES
        KmH  = (kbH + kpfH) / kfH;    % uM, Apparent Michaelis const for S to P
 
        VfmaxH = Etot*kpfH;           % uM/s, Ostensible Vmax if only one substrate
        Kmbh  = (kbH + kpfH)/kpbH;    % uM, Apparent Km for X --> H back reaction
        VbmaxH = Etot*kbH;            % uM/s, Ostensible VbmaxH for back reaction

        kbX  = KdX * kfX;             % 1/s, Backwards reaction ES to S + E
        kpbX = kpfX / KpX;            % 1/(uM*s), Backwards reaction P + E to ES
        KmX  = (kbX + kpfX) / kfX;    % uM, Apparent Michaelis const for S to P

        VfmaxX = Etot*kpfX;           % uM/s, Ostensible Vmax if only one substrate
        Kmbx  = (kbX + kpfX)/kpbX;    % uM, Apparent Km for U --> X back reaction
        VbmaxX = Etot*kbX;            % uM/s, Ostensible Vmaxbx for back reaction

  save CommonParamsCalc.mat kfX kbX kpfX kpbX kbH kfH kpfH kpbH Etot   

 % VARIABLE CONCENTRATIONS
       % E(t) uM;           % Enzyme
       % EHx(t) uM;         % Enzyme-Hx complex
        EHx0 = 0; % uM;	    % Enzyme-Hx complex Initial condition
       % EXa(t) uM,         % Enzyme-Xa complex
        EXa0 = 0;% uM;      % Enzyme-Xa complex and Initial condition
       % Hx(t) uM;  
        Hx0  = 47; % uM;    % Substrate and Initial condition
       % Xa(t) uM;          % Product
        Xa0  = 0; % uM;     % Product, first and Initial condition
       % Ua(t) uM;          % Product, second, Uric acid
        Ua0  = 0; % uM;     % Product, second and Initial condition
       % Uacons(t) uM;      % Uric acid calculated by conservation, as a check

       options = odeset('RelTol',1e-8, 'AbsTol',1e-10, 'MaxStep',1e-2, 'InitialStep',1e-2,...
    'NonNegative',(1:5), 'MaxOrder',5, 'BDF','on', 'Stats','off');
       
    % INITIAL CONDITIONS
    %    when (t=t.min){ Hx= Hx0; EHx = EHx0; Xa = Xa0; EXa = EXa0; Ua = Ua0; }
        Y0 = [ EHx0; EXa0; Hx0; Xa0; Ua0; ];
    
    % Solve variables:
    [T,Y] = ode15s(@funsys,[0:0.1:100],Y0,options);      
       EHx = Y(:,1); EXa = Y(:,2); Hx = Y(:,3); Xa = Y(:,4); Ua = Y(:,5);
 
    % Efree  = Etot - EHx - EXa;        % defines free E, uM 
       Efree = zeros(length(T),1);
       for i = 1:length(T)
           [~,Efree(i,:)] = funsys(T(i),Y(i,:));
       end
            
    % Uacons(t) = Hx0 + Xa0 + Ua0 - Hx - Xa - EHx - EXa; % Test: Uacons should = Ua
      Uacons = zeros(length(T),1); % uM
      for i = 1:length(T)
          Uacons(i) = Hx0 + Xa0 + Ua0 - Hx(i) - Xa(i) - EHx(i) - EXa(i);
      end
      
      

%% Load datasets and compare with model results:
    cost_func = 'NRMSE';

    HypoData = csvread(fileHypo);
    HypoResults = [T,Hx];        % Model results for hypoxanthine
    T_hypoD = HypoData(:,1);
    Hypo_ConcD = HypoData(:,2);
    HypoR_Match = zeros(length(T_hypoD),2); % Contains model results at data time points.
    for y = 1:length(T_hypoD)
        for z = 1:length(HypoResults)
            if abs(T_hypoD(y)-T(z))<0.001
                HypoR_Match(y,1:2) = HypoResults(z,1:2);
            end
        end
    end
    fitHypo = goodnessOfFit(HypoR_Match,HypoData,cost_func);

 %  Xanthine
    XanData = csvread(fileXan);
    XanResults = [T,Xa];        % Model results
    T_xanD = XanData(:,1);
    Xan_ConcD = XanData(:,2);
    XanR_Match = zeros(length(T_xanD),2); % Contains model results at data time points.
    for y = 1:length(T_xanD)
        for z = 1:length(XanResults)
            if abs(T_xanD(y)-T(z))<0.001
                XanR_Match(y,1:2) = XanResults(z,1:2);
            end
        end
    end
    fitXan = goodnessOfFit(XanR_Match,XanData,cost_func);
  
  % Uric acid  
    UricData = csvread(fileUric);
    UricResults = [T,Ua];        % Model results
    T_uricD = UricData(:,1);
    Uric_ConcD = UricData(:,2);
    UricR_Match = zeros(length(T_uricD),2); % Contains model results at data time points.
    for y = 1:length(T_uricD)
        for z = 1:length(UricResults)
            if abs(T_uricD(y)-T(z))<0.001
                UricR_Match(y,1:2) = UricResults(z,1:2);
            end
        end
    end
    fitUric = goodnessOfFit(UricR_Match,UricData,cost_func);
    
    Fitness = (fitHypo(2) + fitXan(2) + fitUric(2))/3;   % 1 is perfect, -inf is very bad  
    
%% Plot various model simulation results 
figure(1); hold on; box on;
h_fig1 = figure(1);
h_fig1.Name = 'FullXO: Hyp ->Xa -> UA';
plot(T,Hx,'-k', T,Xa,'-r', T,Ua,'-b',HypoData(:,1),HypoData(:,2),'-.gx',T_xanD,Xan_ConcD,'-.cx',T_uricD,Uric_ConcD,'-.mx','LineWidth',1)
legend('Hypoxanthine','Xanthine','Uric acid','Hypo data', 'Xan data', 'Uric data')
axis([0 100 0 50])
set(gca,'LineWidth',1.5,'FontName','Arial','FontSize',16);
xlabel('Time, sec'); ylabel('Conc, uM');

figure(2); hold on; box on;
h_fig1 = figure(2);
h_fig1.Name = 'Free and bound enzyme conc: Hyp ->Xa -> UA';
plot(T,ones(size(T))*Etot,'-r', T,EHx,'-b',T,EXa,'-g',T,Efree,'-k','LineWidth',1)
legend('Total Enzyme','EHx','EXa','E free')
axis([0 100 0 1.2])
set(gca,'LineWidth',1.5,'FontName','Arial','FontSize',16);
xlabel('Time, sec'); ylabel('Conc, uM');


%  kfX        100      3.113E4         offrate  = KdX*kfX = 100*4.9 = 490/s
%  KdX       4.962      2.117         Apparent KmX = (kbX +kpfX)/kfX
%                                                  = (490 +0.71)/100 = 4.91 uM
%  kpfX     0.7133      .0095     Ostensible VmaxX = Etot*kpfX = 2.7*0.71 uM/s. assuming kfX to be instantaneous, but it is in series with the binding process kfX and is slowed by it =1.92 uM/s.
%  VmaxbH = (kbH + kpfH)/kpbH
%  VmaxbX = (kbX + kpfX)/kpbX

%  FIGURE:
%    This model represents a series of single enzyme reversible reactions for
%    the sequence: Hx <-> Xa <-> U as for  Hx, Xa, and UA in endothelial cells
%   
%                      kfH -->             kpfH -->            KdH = kbH/kfH
%           Hx + E  <----------->  EHx  <----------->  Xa + E
%                      <-- kbH            <-- kpbH             Kp1 = kpb1/kpf1


%                      kfX -->             kpfX -->            KdX = kbX/kfX
%           Xa + E  <----------->  EXa  <----------->  Ua + E
%                      <-- kbX            <-- kpbX             KpX = kpbX/kpfX

%     The step Xa <-> Ua uses the same enzyme as does Hx <-> Xa, xanthine oxidase.
%     For enzyme, it's occupancy is by either Hx or Xa but not both. The code
%     allows different affinities for the two substrates and different binding rates.
      
% where:

%VfmaxH = Etot * kpfH         % forward Vmax
% VbmaxH = Etot * kbH          % backward Vmax
% KmH = (kbH + kpfH) / (kfH)   % Km for forward Hx reaction
% KpH = kpfH / kpbH            % Km for backward Hx reaction  
% VfmaxX = Etot * kpfX         % forward Vmax
% VbmaxX = Etot * kbX          % backward Vmax
% KmX = (kbX + kpfX) / (kfX)   % Km for forward Xa reaction
% KpX = kpfX / kpbX            % Km for backward Xa reaction
 

% DETAILED DESCRIPTION: Bidirectional fluxes Hx <--> Xa <--> Ua facilitated 
% by a single enzyme, Xanthine Oxidase (EC# 1.7.3.2), in a hyperoxic medium at pH 8,
% so it is oxidative. The equations are standard reactions, forward and backward, so 
% the concentration changes are driven by the NET flux through each reaction. The
% enzyme is assumed to bind only 1 substrate at a time, so all three of Hx, Xa and Ua 
% compete for the single site.

% A conservation test is provided by:
% Uacons = Hx0 + Xa0 + Ua0 - Hx - Xa - EHx - EXa; 
% where the Test is that Uacons should = Ua.

% SHORTCOMINGS/GENERAL COMMENTS:
%   The model assumes high rates of reversibility ES <--> EP, considering them as 
%   the same complex and does not define any mechanism for their interconversion.

% REVISION HISTORY:
%	Original 28dec09 by JB Bassingthwaighte derived from Model 130
%	Revised  30dec09 by B Jardine  Update format and correct spelling
%                 18apr10 by JBB to clarify comments and clean format
%                 19nov11 by jbb adapt to Xa and add data. 
%                 28jun12 by BEJ, add keyword reproducible.
%                 24dec12 by BEJ, rename variables to be consistent with paper 
%                 18sep2018  BEJ: Convert to Matlab

% COPYRIGHT AND REQUEST FOR ACKNOWLEDGMENT OF USE:   
%  Copyright (C) 1999-2018 University of Washington. From the National Simulation Resource,  
%  Director J. B. Bassingthwaighte, Department of Bioengineering, University of Washington, Seattle WA 98195-5061. 
%  Academic use is unrestricted. Software may be copied so long as this copyright notice is included.

%  When citing JSim please use this reference: Butterworth E, Jardine BE, Raymond GM, Neal ML, Bassingthwaighte JB. 
%  JSim, an open-source modeling system for data analysis [v3; ref status: indexed, http://f1000r.es/3n0] 
%  F1000Research 2014, 2:288 (doi: 10.12688/f1000research.2-288.v3)  

%  This software was developed with support from NIH grants HL088516 and HL073598, NIBIB grant BE08417,
%  the Cardiac Energy Grid HL122199 (PI: J.B. Bassingthwaighte), and the Virtual Physiological Rat program 
%  GM094503 (PI: D.A.Beard). Please cite these grants in any publication for which this software is used and 
%  send an email with the citation and, if possible, a PDF file of the paper to: staff@physiome.org. 

